@@ -100,13 +100,13 @@ type EntryPoint struct {
 type LinkerGraph struct {
 	Files       []LinkerFile
 	entryPoints []EntryPoint
-	Symbols     js_ast.SymbolMap
+	Symbols     ast.SymbolMap
 
 	// This is for cross-module inlining of TypeScript enum constants
-	TSEnums map[js_ast.Ref]map[string]js_ast.TSEnumValue
+	TSEnums map[ast.Ref]map[string]js_ast.TSEnumValue
 
 	// This is for cross-module inlining of detected inlinable constants
-	ConstValues map[js_ast.Ref]js_ast.ConstValue
+	ConstValues map[ast.Ref]js_ast.ConstValue
 
 	// We should avoid traversing all files in the bundle, because the linker
 	// should be able to run a linking operation on a large bundle where only
@@ -119,7 +119,7 @@ type LinkerGraph struct {
 
 	// This maps from unstable source index to stable reachable file index. This
 	// is useful as a deterministic key for sorting if you need to sort something
-	// containing a source index (such as "js_ast.Ref" symbol references).
+	// containing a source index (such as "ast.Ref" symbol references).
 	StableSourceIndices []uint32
 }
 
@@ -130,7 +130,7 @@ func CloneLinkerGraph(
 	codeSplitting bool,
 ) LinkerGraph {
 	entryPoints := append([]EntryPoint{}, originalEntryPoints...)
-	symbols := js_ast.NewSymbolMap(len(inputFiles))
+	symbols := ast.NewSymbolMap(len(inputFiles))
 	files := make([]LinkerFile, len(inputFiles))
 
 	// Mark all entry points so we don't add them again for import() expressions
@@ -164,7 +164,7 @@ func CloneLinkerGraph(
 				}
 
 				// Clone the symbol map
-				fileSymbols := append([]js_ast.Symbol{}, repr.AST.Symbols...)
+				fileSymbols := append([]ast.Symbol{}, repr.AST.Symbols...)
 				symbols.SymbolsForSource[sourceIndex] = fileSymbols
 				repr.AST.Symbols = nil
 
@@ -172,7 +172,7 @@ func CloneLinkerGraph(
 				repr.AST.Parts = append([]js_ast.Part{}, repr.AST.Parts...)
 				for i := range repr.AST.Parts {
 					part := &repr.AST.Parts[i]
-					clone := make(map[js_ast.Ref]js_ast.SymbolUse, len(part.SymbolUses))
+					clone := make(map[ast.Ref]js_ast.SymbolUse, len(part.SymbolUses))
 					for ref, uses := range part.SymbolUses {
 						clone[ref] = uses
 					}
@@ -195,13 +195,13 @@ func CloneLinkerGraph(
 							// For example, "import('./foo.json', { assert: { type: 'json' } })"
 							// will likely be converted into an import of a JavaScript file and
 							// leaving the import assertion there will prevent it from working.
-							record.Assertions = nil
+							record.AssertOrWith = nil
 						}
 					}
 				}
 
 				// Clone the import map
-				namedImports := make(map[js_ast.Ref]js_ast.NamedImport, len(repr.AST.NamedImports))
+				namedImports := make(map[ast.Ref]js_ast.NamedImport, len(repr.AST.NamedImports))
 				for k, v := range repr.AST.NamedImports {
 					namedImports[k] = v
 				}
@@ -221,14 +221,14 @@ func CloneLinkerGraph(
 				{
 					new := &js_ast.Scope{}
 					*new = *repr.AST.ModuleScope
-					new.Generated = append([]js_ast.Ref{}, new.Generated...)
+					new.Generated = append([]ast.Ref{}, new.Generated...)
 					repr.AST.ModuleScope = new
 				}
 
 				// Also associate some default metadata with the file
 				repr.Meta.ResolvedExports = resolvedExports
-				repr.Meta.IsProbablyTypeScriptType = make(map[js_ast.Ref]bool)
-				repr.Meta.ImportsToBind = make(map[js_ast.Ref]ImportData)
+				repr.Meta.IsProbablyTypeScriptType = make(map[ast.Ref]bool)
+				repr.Meta.ImportsToBind = make(map[ast.Ref]ImportData)
 
 			case *CSSRepr:
 				// Clone the representation
@@ -238,6 +238,11 @@ func CloneLinkerGraph(
 					file.InputFile.Repr = repr
 				}
 
+				// Clone the symbol map
+				fileSymbols := append([]ast.Symbol{}, repr.AST.Symbols...)
+				symbols.SymbolsForSource[sourceIndex] = fileSymbols
+				repr.AST.Symbols = nil
+
 				// Clone the import records
 				repr.AST.ImportRecords = append([]ast.ImportRecord{}, repr.AST.ImportRecords...)
 			}
@@ -265,8 +270,8 @@ func CloneLinkerGraph(
 	}
 
 	// Do a final quick pass over all files
-	var tsEnums map[js_ast.Ref]map[string]js_ast.TSEnumValue
-	var constValues map[js_ast.Ref]js_ast.ConstValue
+	var tsEnums map[ast.Ref]map[string]js_ast.TSEnumValue
+	var constValues map[ast.Ref]js_ast.ConstValue
 	bitCount := uint(len(entryPoints))
 	for _, sourceIndex := range reachableFiles {
 		file := &files[sourceIndex]
@@ -279,7 +284,7 @@ func CloneLinkerGraph(
 		// it should be fine to just merge them together in serial.
 		if repr, ok := file.InputFile.Repr.(*JSRepr); ok && repr.AST.TSEnums != nil {
 			if tsEnums == nil {
-				tsEnums = make(map[js_ast.Ref]map[string]js_ast.TSEnumValue)
+				tsEnums = make(map[ast.Ref]map[string]js_ast.TSEnumValue)
 			}
 			for ref, enum := range repr.AST.TSEnums {
 				tsEnums[ref] = enum
@@ -289,7 +294,7 @@ func CloneLinkerGraph(
 		// Also merge const values into one big map as well
 		if repr, ok := file.InputFile.Repr.(*JSRepr); ok && repr.AST.ConstValues != nil {
 			if constValues == nil {
-				constValues = make(map[js_ast.Ref]js_ast.ConstValue)
+				constValues = make(map[ast.Ref]js_ast.ConstValue)
 			}
 			for ref, value := range repr.AST.ConstValues {
 				constValues[ref] = value
@@ -316,7 +321,7 @@ func (g *LinkerGraph) EntryPoints() []EntryPoint {
 func (g *LinkerGraph) AddPartToFile(sourceIndex uint32, part js_ast.Part) uint32 {
 	// Invariant: this map is never null
 	if part.SymbolUses == nil {
-		part.SymbolUses = make(map[js_ast.Ref]js_ast.SymbolUse)
+		part.SymbolUses = make(map[ast.Ref]js_ast.SymbolUse)
 	}
 
 	repr := g.Files[sourceIndex].InputFile.Repr.(*JSRepr)
@@ -337,7 +342,7 @@ func (g *LinkerGraph) AddPartToFile(sourceIndex uint32, part js_ast.Part) uint32
 			// Add this part to the overlay
 			partIndices = append(partIndices, partIndex)
 			if repr.Meta.TopLevelSymbolToPartsOverlay == nil {
-				repr.Meta.TopLevelSymbolToPartsOverlay = make(map[js_ast.Ref][]uint32)
+				repr.Meta.TopLevelSymbolToPartsOverlay = make(map[ast.Ref][]uint32)
 			}
 			repr.Meta.TopLevelSymbolToPartsOverlay[declaredSymbol.Ref] = partIndices
 		}
@@ -346,18 +351,18 @@ func (g *LinkerGraph) AddPartToFile(sourceIndex uint32, part js_ast.Part) uint32
 	return partIndex
 }
 
-func (g *LinkerGraph) GenerateNewSymbol(sourceIndex uint32, kind js_ast.SymbolKind, originalName string) js_ast.Ref {
+func (g *LinkerGraph) GenerateNewSymbol(sourceIndex uint32, kind ast.SymbolKind, originalName string) ast.Ref {
 	sourceSymbols := &g.Symbols.SymbolsForSource[sourceIndex]
 
-	ref := js_ast.Ref{
+	ref := ast.Ref{
 		SourceIndex: sourceIndex,
 		InnerIndex:  uint32(len(*sourceSymbols)),
 	}
 
-	*sourceSymbols = append(*sourceSymbols, js_ast.Symbol{
+	*sourceSymbols = append(*sourceSymbols, ast.Symbol{
 		Kind:         kind,
 		OriginalName: originalName,
-		Link:         js_ast.InvalidRef,
+		Link:         ast.InvalidRef,
 	})
 
 	generated := &g.Files[sourceIndex].InputFile.Repr.(*JSRepr).AST.ModuleScope.Generated
@@ -368,7 +373,7 @@ func (g *LinkerGraph) GenerateNewSymbol(sourceIndex uint32, kind js_ast.SymbolKi
 func (g *LinkerGraph) GenerateSymbolImportAndUse(
 	sourceIndex uint32,
 	partIndex uint32,
-	ref js_ast.Ref,
+	ref ast.Ref,
 	useCount uint32,
 	sourceIndexToImportFrom uint32,
 ) {
