@@ -501,7 +501,7 @@ func ConvertBindingToExpr(binding Binding, wrapIdentifier func(logger.Loc, ast.R
 		properties := make([]Property, len(b.Properties))
 		for i, property := range b.Properties {
 			value := ConvertBindingToExpr(property.Value, wrapIdentifier)
-			kind := PropertyNormal
+			kind := PropertyField
 			if property.IsSpread {
 				kind = PropertySpread
 			}
@@ -946,14 +946,12 @@ func ToUint32(f float64) uint32 {
 	return uint32(ToInt32(f))
 }
 
+// If this returns true, we know the result can't be NaN
 func isInt32OrUint32(data E) bool {
 	switch e := data.(type) {
-	case *EUnary:
-		return e.Op == UnOpCpl
-
 	case *EBinary:
 		switch e.Op {
-		case BinOpBitwiseAnd, BinOpBitwiseOr, BinOpBitwiseXor, BinOpShl, BinOpShr, BinOpUShr:
+		case BinOpUShr: // This is the only bitwise operator that can't return a bigint (because it throws instead)
 			return true
 
 		case BinOpLogicalOr, BinOpLogicalAnd:
@@ -1133,9 +1131,15 @@ func approximatePrintedIntCharCount(intValue float64) int {
 	return count
 }
 
-func ShouldFoldBinaryArithmeticWhenMinifying(binary *EBinary) bool {
+func ShouldFoldBinaryOperatorWhenMinifying(binary *EBinary) bool {
 	switch binary.Op {
 	case
+		// Equality tests should always result in smaller code when folded
+		BinOpLooseEq,
+		BinOpLooseNe,
+		BinOpStrictEq,
+		BinOpStrictNe,
+
 		// Minification always folds right signed shift operations since they are
 		// unlikely to result in larger output. Note: ">>>" could result in
 		// bigger output such as "-1 >>> 0" becoming "4294967295".
@@ -1161,6 +1165,11 @@ func ShouldFoldBinaryArithmeticWhenMinifying(binary *EBinary) bool {
 			return true
 		}
 
+		// String addition should pretty much always be more compact when folded
+		if _, _, ok := extractStringValues(binary.Left, binary.Right); ok {
+			return true
+		}
+
 	case BinOpSub:
 		// Subtraction of small-ish integers can definitely be folded without issues
 		// "3 - 1" => "2"
@@ -1197,18 +1206,26 @@ func ShouldFoldBinaryArithmeticWhenMinifying(binary *EBinary) bool {
 			resultLen := approximatePrintedIntCharCount(float64(ToUint32(left) >> (ToUint32(right) & 31)))
 			return resultLen <= leftLen+3+rightLen
 		}
+
+	case BinOpLogicalAnd, BinOpLogicalOr, BinOpNullishCoalescing:
+		if IsPrimitiveLiteral(binary.Left.Data) {
+			return true
+		}
 	}
 	return false
 }
 
 // This function intentionally avoids mutating the input AST so it can be
 // called after the AST has been frozen (i.e. after parsing ends).
-func FoldBinaryArithmetic(loc logger.Loc, e *EBinary) Expr {
+func FoldBinaryOperator(loc logger.Loc, e *EBinary) Expr {
 	switch e.Op {
 	case BinOpAdd:
 		if left, right, ok := extractNumericValues(e.Left, e.Right); ok {
 			return Expr{Loc: loc, Data: &ENumber{Value: left + right}}
 		}
+		if left, right, ok := extractStringValues(e.Left, e.Right); ok {
+			return Expr{Loc: loc, Data: &EString{Value: joinStrings(left, right)}}
+		}
 
 	case BinOpSub:
 		if left, right, ok := extractNumericValues(e.Left, e.Right); ok {
@@ -1296,6 +1313,49 @@ func FoldBinaryArithmetic(loc logger.Loc, e *EBinary) Expr {
 		if left, right, ok := extractStringValues(e.Left, e.Right); ok {
 			return Expr{Loc: loc, Data: &EBoolean{Value: stringCompareUCS2(left, right) >= 0}}
 		}
+
+	case BinOpLooseEq, BinOpStrictEq:
+		if left, right, ok := extractNumericValues(e.Left, e.Right); ok {
+			return Expr{Loc: loc, Data: &EBoolean{Value: left == right}}
+		}
+		if left, right, ok := extractStringValues(e.Left, e.Right); ok {
+			return Expr{Loc: loc, Data: &EBoolean{Value: stringCompareUCS2(left, right) == 0}}
+		}
+
+	case BinOpLooseNe, BinOpStrictNe:
+		if left, right, ok := extractNumericValues(e.Left, e.Right); ok {
+			return Expr{Loc: loc, Data: &EBoolean{Value: left != right}}
+		}
+		if left, right, ok := extractStringValues(e.Left, e.Right); ok {
+			return Expr{Loc: loc, Data: &EBoolean{Value: stringCompareUCS2(left, right) != 0}}
+		}
+
+	case BinOpLogicalAnd:
+		if boolean, sideEffects, ok := ToBooleanWithSideEffects(e.Left.Data); ok {
+			if !boolean {
+				return e.Left
+			} else if sideEffects == NoSideEffects {
+				return e.Right
+			}
+		}
+
+	case BinOpLogicalOr:
+		if boolean, sideEffects, ok := ToBooleanWithSideEffects(e.Left.Data); ok {
+			if boolean {
+				return e.Left
+			} else if sideEffects == NoSideEffects {
+				return e.Right
+			}
+		}
+
+	case BinOpNullishCoalescing:
+		if isNullOrUndefined, sideEffects, ok := ToNullOrUndefinedWithSideEffects(e.Left.Data); ok {
+			if !isNullOrUndefined {
+				return e.Left
+			} else if sideEffects == NoSideEffects {
+				return e.Right
+			}
+		}
 	}
 
 	return Expr{}
@@ -1434,6 +1494,12 @@ func CheckEqualityIfNoSideEffects(left E, right E, kind EqualityKind) (equal boo
 		case *ENull, *EUndefined:
 			// "(not null or undefined) == undefined" is false
 			return false, true
+
+		default:
+			if kind == StrictEquality && IsPrimitiveLiteral(right) {
+				// "boolean === (not boolean)" is false
+				return false, true
+			}
 		}
 
 	case *ENumber:
@@ -1461,6 +1527,12 @@ func CheckEqualityIfNoSideEffects(left E, right E, kind EqualityKind) (equal boo
 		case *ENull, *EUndefined:
 			// "(not null or undefined) == undefined" is false
 			return false, true
+
+		default:
+			if kind == StrictEquality && IsPrimitiveLiteral(right) {
+				// "number === (not number)" is false
+				return false, true
+			}
 		}
 
 	case *EBigInt:
@@ -1473,6 +1545,12 @@ func CheckEqualityIfNoSideEffects(left E, right E, kind EqualityKind) (equal boo
 		case *ENull, *EUndefined:
 			// "(not null or undefined) == undefined" is false
 			return false, true
+
+		default:
+			if kind == StrictEquality && IsPrimitiveLiteral(right) {
+				// "bigint === (not bigint)" is false
+				return false, true
+			}
 		}
 
 	case *EString:
@@ -1485,6 +1563,12 @@ func CheckEqualityIfNoSideEffects(left E, right E, kind EqualityKind) (equal boo
 		case *ENull, *EUndefined:
 			// "(not null or undefined) == undefined" is false
 			return false, true
+
+		default:
+			if kind == StrictEquality && IsPrimitiveLiteral(right) {
+				// "string === (not string)" is false
+				return false, true
+			}
 		}
 	}
 
@@ -2019,10 +2103,10 @@ func (ctx HelperContext) SimplifyBooleanExpr(expr Expr) Expr {
 				// in a boolean context is unnecessary because the value is
 				// only truthy if it's not zero.
 				if e.Op == BinOpStrictNe || e.Op == BinOpLooseNe {
-					// "if ((a | b) !== 0)" => "if (a | b)"
+					// "if ((a >>> b) !== 0)" => "if (a >>> b)"
 					return left
 				} else {
-					// "if ((a | b) === 0)" => "if (!(a | b))"
+					// "if ((a >>> b) === 0)" => "if (!(a >>> b))"
 					return Not(left)
 				}
 			}
@@ -2268,7 +2352,7 @@ func (ctx HelperContext) ClassCanBeRemovedIfUnused(class Class) bool {
 			return false
 		}
 
-		if property.Flags.Has(PropertyIsMethod) {
+		if property.Kind.IsMethodDefinition() {
 			if fn, ok := property.ValueOrNil.Data.(*EFunction); ok {
 				for _, arg := range fn.Fn.Args {
 					if len(arg.Decorators) > 0 {
@@ -2321,7 +2405,7 @@ func (ctx HelperContext) ClassCanBeRemovedIfUnused(class Class) bool {
 			//     static foo = 1
 			//   }
 			//
-			if !class.UseDefineForClassFields && !property.Flags.Has(PropertyIsMethod) {
+			if property.Kind == PropertyField && !class.UseDefineForClassFields {
 				return false
 			}
 		}
@@ -2645,7 +2729,7 @@ func MangleObjectSpread(properties []Property) []Property {
 					// descriptor is not inlined into the caller. Since we are not
 					// evaluating code at compile time, just bail if we hit one
 					// and preserve the spread with the remaining properties.
-					if p.Kind == PropertyGet || p.Kind == PropertySet {
+					if p.Kind == PropertyGetter || p.Kind == PropertySetter {
 						// Don't mutate the original AST
 						clone := *v
 						clone.Properties = v.Properties[i:]
@@ -2657,7 +2741,7 @@ func MangleObjectSpread(properties []Property) []Property {
 					// Also bail if we hit a verbatim "__proto__" key. This will
 					// actually set the prototype of the object being spread so
 					// inlining it is not correct.
-					if p.Kind == PropertyNormal && !p.Flags.Has(PropertyIsComputed) && !p.Flags.Has(PropertyIsMethod) {
+					if p.Kind == PropertyField && !p.Flags.Has(PropertyIsComputed) {
 						if str, ok := p.Key.Data.(*EString); ok && helpers.UTF16EqualsString(str.Value, "__proto__") {
 							// Don't mutate the original AST
 							clone := *v
