@@ -138,7 +138,7 @@ func (p *jsonParser) parseExpr() js_ast.Expr {
 			value := p.parseExpr()
 
 			property := js_ast.Property{
-				Kind:       js_ast.PropertyNormal,
+				Kind:       js_ast.PropertyField,
 				Loc:        keyRange.Loc,
 				Key:        key,
 				ValueOrNil: value,
@@ -165,6 +165,14 @@ func (p *jsonParser) parseExpr() js_ast.Expr {
 			CloseBraceLoc: closeBraceLoc,
 		}}
 
+	case js_lexer.TBigIntegerLiteral:
+		if !p.options.IsForDefine {
+			p.lexer.Unexpected()
+		}
+		value := p.lexer.Identifier
+		p.lexer.Next()
+		return js_ast.Expr{Loc: loc, Data: &js_ast.EBigInt{Value: value.String}}
+
 	default:
 		p.lexer.Unexpected()
 		return js_ast.Expr{}
@@ -175,6 +183,7 @@ type JSONOptions struct {
 	UnsupportedJSFeatures compat.JSFeature
 	Flavor                js_lexer.JSONFlavor
 	ErrorSuffix           string
+	IsForDefine           bool
 }
 
 func ParseJSON(log logger.Log, source logger.Source, options JSONOptions) (result js_ast.Expr, ok bool) {
@@ -221,7 +230,7 @@ func isValidJSON(value js_ast.Expr) bool {
 
 	case *js_ast.EObject:
 		for _, property := range e.Properties {
-			if property.Kind != js_ast.PropertyNormal || property.Flags&(js_ast.PropertyIsComputed|js_ast.PropertyIsMethod) != 0 {
+			if property.Kind != js_ast.PropertyField || property.Flags.Has(js_ast.PropertyIsComputed) {
 				return false
 			}
 			if _, ok := property.Key.Data.(*js_ast.EString); !ok {
