@@ -22,6 +22,9 @@ func expectPrintedCommon(t *testing.T, name string, contents string, expected st
 		msgs := log.Done()
 		text := ""
 		for _, msg := range msgs {
+			if msg.Kind != logger.Error {
+				continue
+			}
 			text += msg.String(logger.OutputOptions{}, logger.TerminalInfo{})
 		}
 		test.AssertEqualWithDiff(t, text, "")
@@ -517,26 +520,32 @@ func TestObject(t *testing.T) {
 	expectPrintedTarget(t, 5, "function foo(__proto__) { return { __proto__ } }", "function foo(__proto__) {\n  return { __proto__: __proto__ };\n}\n")
 }
 
+func TestSwitch(t *testing.T) {
+	// Ideally comments on case clauses would be preserved
+	expectPrinted(t, "switch (x) { /* 1 */ case 1: /* 2 */ case 2: /* default */ default: break }",
+		"switch (x) {\n  /* 1 */\n  case 1:\n  /* 2 */\n  case 2:\n  /* default */\n  default:\n    break;\n}\n")
+}
+
 func TestFor(t *testing.T) {
 	// Make sure "in" expressions are forbidden in the right places
-	expectPrinted(t, "for ((a in b);;);", "for ((a in b); ; )\n  ;\n")
-	expectPrinted(t, "for (a ? b : (c in d);;);", "for (a ? b : (c in d); ; )\n  ;\n")
-	expectPrinted(t, "for ((a ? b : c in d).foo;;);", "for ((a ? b : c in d).foo; ; )\n  ;\n")
-	expectPrinted(t, "for (var x = (a in b);;);", "for (var x = (a in b); ; )\n  ;\n")
-	expectPrinted(t, "for (x = (a in b);;);", "for (x = (a in b); ; )\n  ;\n")
-	expectPrinted(t, "for (x == (a in b);;);", "for (x == (a in b); ; )\n  ;\n")
-	expectPrinted(t, "for (1 * (x == a in b);;);", "for (1 * (x == a in b); ; )\n  ;\n")
-	expectPrinted(t, "for (a ? b : x = (c in d);;);", "for (a ? b : x = (c in d); ; )\n  ;\n")
-	expectPrinted(t, "for (var x = y = (a in b);;);", "for (var x = y = (a in b); ; )\n  ;\n")
-	expectPrinted(t, "for ([a in b];;);", "for ([a in b]; ; )\n  ;\n")
-	expectPrinted(t, "for (x(a in b);;);", "for (x(a in b); ; )\n  ;\n")
-	expectPrinted(t, "for (x[a in b];;);", "for (x[a in b]; ; )\n  ;\n")
-	expectPrinted(t, "for (x?.[a in b];;);", "for (x?.[a in b]; ; )\n  ;\n")
-	expectPrinted(t, "for ((x => a in b);;);", "for ((x) => (a in b); ; )\n  ;\n")
+	expectPrinted(t, "for ((a in b);;);", "for ((a in b); ; ) ;\n")
+	expectPrinted(t, "for (a ? b : (c in d);;);", "for (a ? b : (c in d); ; ) ;\n")
+	expectPrinted(t, "for ((a ? b : c in d).foo;;);", "for ((a ? b : c in d).foo; ; ) ;\n")
+	expectPrinted(t, "for (var x = (a in b);;);", "for (var x = (a in b); ; ) ;\n")
+	expectPrinted(t, "for (x = (a in b);;);", "for (x = (a in b); ; ) ;\n")
+	expectPrinted(t, "for (x == (a in b);;);", "for (x == (a in b); ; ) ;\n")
+	expectPrinted(t, "for (1 * (x == a in b);;);", "for (1 * (x == a in b); ; ) ;\n")
+	expectPrinted(t, "for (a ? b : x = (c in d);;);", "for (a ? b : x = (c in d); ; ) ;\n")
+	expectPrinted(t, "for (var x = y = (a in b);;);", "for (var x = y = (a in b); ; ) ;\n")
+	expectPrinted(t, "for ([a in b];;);", "for ([a in b]; ; ) ;\n")
+	expectPrinted(t, "for (x(a in b);;);", "for (x(a in b); ; ) ;\n")
+	expectPrinted(t, "for (x[a in b];;);", "for (x[a in b]; ; ) ;\n")
+	expectPrinted(t, "for (x?.[a in b];;);", "for (x?.[a in b]; ; ) ;\n")
+	expectPrinted(t, "for ((x => a in b);;);", "for ((x) => (a in b); ; ) ;\n")
 
 	// Make sure for-of loops with commas are wrapped in parentheses
-	expectPrinted(t, "for (let a in b, c);", "for (let a in b, c)\n  ;\n")
-	expectPrinted(t, "for (let a of (b, c));", "for (let a of (b, c))\n  ;\n")
+	expectPrinted(t, "for (let a in b, c);", "for (let a in b, c) ;\n")
+	expectPrinted(t, "for (let a of (b, c));", "for (let a of (b, c)) ;\n")
 }
 
 func TestFunction(t *testing.T) {
@@ -568,8 +577,8 @@ func TestCommentsAndParentheses(t *testing.T) {
 	expectPrinted(t, "export default (/* foo */ function f() {});", "export default (\n  /* foo */\n  function f() {\n  }\n);\n")
 	expectPrinted(t, "export default (/* foo */ class x {});", "export default (\n  /* foo */\n  class x {\n  }\n);\n")
 	expectPrinted(t, "x = () => (/* foo */ {});", "x = () => (\n  /* foo */\n  {}\n);\n")
-	expectPrinted(t, "for ((/* foo */ let).x of y) ;", "for (\n  /* foo */\n  (let).x of y\n)\n  ;\n")
-	expectPrinted(t, "for (/* foo */ (let).x of y) ;", "for (\n  /* foo */\n  (let).x of y\n)\n  ;\n")
+	expectPrinted(t, "for ((/* foo */ let).x of y) ;", "for (\n  /* foo */\n  (let).x of y\n) ;\n")
+	expectPrinted(t, "for (/* foo */ (let).x of y) ;", "for (\n  /* foo */\n  (let).x of y\n) ;\n")
 	expectPrinted(t, "function *x() { yield (/* foo */ y) }", "function* x() {\n  yield (\n    /* foo */\n    y\n  );\n}\n")
 }
 
@@ -1144,3 +1153,15 @@ func TestUsing(t *testing.T) {
 	expectPrintedMinify(t, "await using x = y", "await using x=y;")
 	expectPrintedMinify(t, "await using x = y, z = _", "await using x=y,z=_;")
 }
+
+func TestMinifyBigInt(t *testing.T) {
+	expectPrintedTargetMangle(t, 2019, "x = 0b100101n", "x = /* @__PURE__ */ BigInt(37);\n")
+	expectPrintedTargetMangle(t, 2019, "x = 0B100101n", "x = /* @__PURE__ */ BigInt(37);\n")
+	expectPrintedTargetMangle(t, 2019, "x = 0o76543210n", "x = /* @__PURE__ */ BigInt(16434824);\n")
+	expectPrintedTargetMangle(t, 2019, "x = 0O76543210n", "x = /* @__PURE__ */ BigInt(16434824);\n")
+	expectPrintedTargetMangle(t, 2019, "x = 0xFEDCBA9876543210n", "x = /* @__PURE__ */ BigInt(\"0xFEDCBA9876543210\");\n")
+	expectPrintedTargetMangle(t, 2019, "x = 0XFEDCBA9876543210n", "x = /* @__PURE__ */ BigInt(\"0XFEDCBA9876543210\");\n")
+	expectPrintedTargetMangle(t, 2019, "x = 0xb0ba_cafe_f00dn", "x = /* @__PURE__ */ BigInt(0xb0bacafef00d);\n")
+	expectPrintedTargetMangle(t, 2019, "x = 0xB0BA_CAFE_F00Dn", "x = /* @__PURE__ */ BigInt(0xB0BACAFEF00D);\n")
+	expectPrintedTargetMangle(t, 2019, "x = 102030405060708090807060504030201n", "x = /* @__PURE__ */ BigInt(\"102030405060708090807060504030201\");\n")
+}
