@@ -11,6 +11,7 @@ package logger
 // default.
 
 import (
+	"encoding/binary"
 	"fmt"
 	"os"
 	"runtime"
@@ -185,6 +186,21 @@ func (r Range) End() int32 {
 	return r.Loc.Start + r.Len
 }
 
+func (a *Range) ExpandBy(b Range) {
+	if a.Len == 0 {
+		*a = b
+	} else {
+		end := a.End()
+		if n := b.End(); n > end {
+			end = n
+		}
+		if b.Loc.Start < a.Loc.Start {
+			a.Loc.Start = b.Loc.Start
+		}
+		a.Len = end - a.Loc.Start
+	}
+}
+
 type Span struct {
 	Text  string
 	Range Range
@@ -233,9 +249,79 @@ type Path struct {
 	// the output. This is supported by other bundlers, so we also support this.
 	IgnoredSuffix string
 
+	// Import attributes (the "with" keyword after an import) can affect path
+	// resolution. In other words, two paths in the same file that are otherwise
+	// equal but that have different import attributes may resolve to different
+	// paths.
+	ImportAttributes ImportAttributes
+
 	Flags PathFlags
 }
 
+// We rely on paths as map keys. Go doesn't support custom hash codes and
+// only implements hash codes for certain types. In particular, hash codes
+// are implemented for strings but not for arrays of strings. So we have to
+// pack these import attributes into a string.
+type ImportAttributes struct {
+	packedData string
+}
+
+type ImportAttribute struct {
+	Key   string
+	Value string
+}
+
+// This returns a sorted array instead of a map to make determinism easier
+func (attrs ImportAttributes) DecodeIntoArray() (result []ImportAttribute) {
+	if attrs.packedData == "" {
+		return nil
+	}
+	bytes := []byte(attrs.packedData)
+	for len(bytes) > 0 {
+		kn := 4 + binary.LittleEndian.Uint32(bytes[:4])
+		k := string(bytes[4:kn])
+		bytes = bytes[kn:]
+		vn := 4 + binary.LittleEndian.Uint32(bytes[:4])
+		v := string(bytes[4:vn])
+		bytes = bytes[vn:]
+		result = append(result, ImportAttribute{Key: k, Value: v})
+	}
+	return result
+}
+
+func (attrs ImportAttributes) DecodeIntoMap() (result map[string]string) {
+	if array := attrs.DecodeIntoArray(); len(array) > 0 {
+		result = make(map[string]string, len(array))
+		for _, attr := range array {
+			result[attr.Key] = attr.Value
+		}
+	}
+	return
+}
+
+func EncodeImportAttributes(value map[string]string) ImportAttributes {
+	if len(value) == 0 {
+		return ImportAttributes{}
+	}
+	keys := make([]string, 0, len(value))
+	for k := range value {
+		keys = append(keys, k)
+	}
+	sort.Strings(keys)
+	var sb strings.Builder
+	var n [4]byte
+	for _, k := range keys {
+		v := value[k]
+		binary.LittleEndian.PutUint32(n[:], uint32(len(k)))
+		sb.Write(n[:])
+		sb.WriteString(k)
+		binary.LittleEndian.PutUint32(n[:], uint32(len(v)))
+		sb.Write(n[:])
+		sb.WriteString(v)
+	}
+	return ImportAttributes{packedData: sb.String()}
+}
+
 type PathFlags uint8
 
 const (
@@ -247,13 +333,6 @@ func (p Path) IsDisabled() bool {
 	return (p.Flags & PathDisabled) != 0
 }
 
-func (a Path) ComesBeforeInSortedOrder(b Path) bool {
-	return a.Namespace > b.Namespace ||
-		(a.Namespace == b.Namespace && (a.Text < b.Text ||
-			(a.Text == b.Text && (a.Flags < b.Flags ||
-				(a.Flags == b.Flags && a.IgnoredSuffix < b.IgnoredSuffix)))))
-}
-
 var noColorResult bool
 var noColorOnce sync.Once
 
@@ -308,7 +387,18 @@ func PlatformIndependentPathDirBaseExt(path string) (dir string, base string, ex
 
 	// Strip off the extension
 	if dot := strings.LastIndexByte(base, '.'); dot >= 0 {
-		base, ext = base[:dot], base[dot:]
+		ext = base[dot:]
+
+		// We default to the "local-css" loader for ".module.css" files. Make sure
+		// the string names generated by this don't all have "_module_" in them.
+		if ext == ".css" {
+			if dot2 := strings.LastIndexByte(base[:dot], '.'); dot2 >= 0 && base[dot2:] == ".module.css" {
+				dot = dot2
+				ext = base[dot:]
+			}
+		}
+
+		base = base[:dot]
 	}
 	return
 }
@@ -1525,12 +1615,14 @@ func detailStruct(data MsgData, terminalInfo TerminalInfo, maxMargin int) MsgDet
 	// Only highlight the first line of the line text
 	loc := *data.Location
 	endOfFirstLine := len(loc.LineText)
-	for i, c := range loc.LineText {
-		if c == '\r' || c == '\n' || c == '\u2028' || c == '\u2029' {
-			endOfFirstLine = i
-			break
-		}
+
+	// Note: This uses "IndexByte" because Go implements this with SIMD, which
+	// can matter a lot for really long lines. Some people pass huge >100mb
+	// minified files as line text for the log message.
+	if i := strings.IndexByte(loc.LineText, '\n'); i >= 0 {
+		endOfFirstLine = i
 	}
+
 	firstLine := loc.LineText[:endOfFirstLine]
 	afterFirstLine := loc.LineText[endOfFirstLine:]
 	if afterFirstLine != "" && !strings.HasSuffix(afterFirstLine, "\n") {
