@@ -1,5 +1,7 @@
 package resolver
 
+// This file implements the Yarn PnP specification: https://yarnpkg.com/advanced/pnp-spec/
+
 import (
 	"fmt"
 	"regexp"
@@ -12,8 +14,6 @@ import (
 	"github.com/evanw/esbuild/internal/logger"
 )
 
-// This file implements the Yarn PnP specification: https://yarnpkg.com/advanced/pnp-spec/
-
 type pnpData struct {
 	// Keys are the package idents, values are sets of references. Combining the
 	// ident with each individual reference yields the set of affected locators.
@@ -204,7 +204,7 @@ func (r resolverQuery) resolveToUnqualified(specifier string, parentURL string,
 			}
 
 			// If parentLocator isn't in manifest.fallbackExclusionList, then
-			if set, _ := manifest.fallbackExclusionList[parentLocator.ident]; !set[parentLocator.reference] {
+			if set := manifest.fallbackExclusionList[parentLocator.ident]; !set[parentLocator.reference] {
 				// Let fallback be RESOLVE_VIA_FALLBACK(manifest, ident)
 				fallback, _ := r.resolveViaFallback(manifest, ident)
 
@@ -302,9 +302,7 @@ func (r resolverQuery) findLocator(manifest *pnpData, moduleUrl string) (pnpIden
 	}
 
 	// The relative path must not start with ./; trim it if needed
-	if strings.HasPrefix(relativeUrl, "./") {
-		relativeUrl = relativeUrl[2:]
-	}
+	relativeUrl = strings.TrimPrefix(relativeUrl, "./")
 
 	// If relativeUrl matches manifest.ignorePatternData, then
 	if manifest.ignorePatternData != nil && manifest.ignorePatternData.MatchString(relativeUrl) {
@@ -370,7 +368,7 @@ func (r resolverQuery) resolveViaFallback(manifest *pnpData, ident string) (pnpI
 	// Let referenceOrAlias be the entry from manifest.fallbackPool referenced by ident
 	referenceOrAlias, ok = manifest.fallbackPool[ident]
 
-	// Return it immediatly, whether it's defined or not
+	// Return it immediately, whether it's defined or not
 	if r.debugLogs != nil {
 		if ok {
 			r.debugLogs.addNote(fmt.Sprintf("    Found fallback for %q in \"fallbackPool\": [%s, %s]", ident,
@@ -618,7 +616,7 @@ func (r resolverQuery) extractYarnPnPDataFromJSON(pnpDataPath string, mode pnpDa
 		if mode == pnpReportErrorsAboutMissingFiles || err != syscall.ENOENT {
 			r.log.AddError(nil, logger.Range{},
 				fmt.Sprintf("Cannot read file %q: %s",
-					r.PrettyPath(logger.Path{Text: pnpDataPath, Namespace: "file"}), err.Error()))
+					PrettyPath(r.fs, logger.Path{Text: pnpDataPath, Namespace: "file"}), err.Error()))
 		}
 		return
 	}
@@ -628,7 +626,7 @@ func (r resolverQuery) extractYarnPnPDataFromJSON(pnpDataPath string, mode pnpDa
 	keyPath := logger.Path{Text: pnpDataPath, Namespace: "file"}
 	source = logger.Source{
 		KeyPath:    keyPath,
-		PrettyPath: r.PrettyPath(keyPath),
+		PrettyPath: PrettyPath(r.fs, keyPath),
 		Contents:   contents,
 	}
 	result, _ = r.caches.JSONCache.Parse(r.log, source, js_parser.JSONOptions{})
@@ -644,7 +642,7 @@ func (r resolverQuery) tryToExtractYarnPnPDataFromJS(pnpDataPath string, mode pn
 		if mode == pnpReportErrorsAboutMissingFiles || err != syscall.ENOENT {
 			r.log.AddError(nil, logger.Range{},
 				fmt.Sprintf("Cannot read file %q: %s",
-					r.PrettyPath(logger.Path{Text: pnpDataPath, Namespace: "file"}), err.Error()))
+					PrettyPath(r.fs, logger.Path{Text: pnpDataPath, Namespace: "file"}), err.Error()))
 		}
 		return
 	}
@@ -655,7 +653,7 @@ func (r resolverQuery) tryToExtractYarnPnPDataFromJS(pnpDataPath string, mode pn
 	keyPath := logger.Path{Text: pnpDataPath, Namespace: "file"}
 	source = logger.Source{
 		KeyPath:    keyPath,
-		PrettyPath: r.PrettyPath(keyPath),
+		PrettyPath: PrettyPath(r.fs, keyPath),
 		Contents:   contents,
 	}
 	ast, _ := r.caches.JSCache.Parse(r.log, source, js_parser.OptionsForYarnPnP())
