@@ -68,7 +68,7 @@ func TestYarnPnP(t *testing.T) {
 			func(current pnpTest) {
 				t.Run(current.It, func(t *testing.T) {
 					fs := fs.MockFS(nil, fs.MockUnix, "/")
-					r := resolverQuery{Resolver: NewResolver(fs, logger.NewDeferLog(logger.DeferLogNoVerboseOrDebug, nil), nil, config.Options{})}
+					r := resolverQuery{Resolver: NewResolver(config.BuildCall, fs, logger.NewDeferLog(logger.DeferLogNoVerboseOrDebug, nil), nil, &config.Options{})}
 					result := r.resolveToUnqualified(current.Imported, current.Importer, manifest)
 
 					var observed string
