@@ -31,11 +31,12 @@ export interface ServeRequest {
   keyfile?: string
   certfile?: string
   fallback?: string
+  corsOrigin?: string[]
 }
 
 export interface ServeResponse {
   port: number
-  host: string
+  hosts: string[]
 }
 
 export interface BuildPlugin {
@@ -170,6 +171,7 @@ export interface ResolveRequest {
   resolveDir?: string
   kind?: string
   pluginData?: number
+  with?: Record<string, string>
 }
 
 export interface ResolveResponse {
@@ -194,6 +196,7 @@ export interface OnResolveRequest {
   resolveDir: string
   kind: types.ImportKind
   pluginData: number
+  with: Record<string, string>
 }
 
 export interface OnResolveResponse {
